"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const injection_handler_1 = require("./injection-handler");
const model_1 = require("../model");
class IoCBindConfig {
    constructor(source, instanceFactory) {
        this.source = source;
        this.instanceFactory = instanceFactory;
    }
    to(target) {
        injection_handler_1.InjectorHandler.checkType(target);
        const targetSource = injection_handler_1.InjectorHandler.getConstructorFromType(target);
        this.targetSource = targetSource;
        if (this.source === targetSource) {
            this.factory((context) => {
                const params = this.getParameters(context);
                const constructor = this.decoratedConstructor || target;
                return this.callConstructor(constructor, params, context);
            });
        }
        else {
            this.factory((context) => {
                return this.instanceFactory(target, context);
            });
        }
        return this;
    }
    callConstructor(constructor, params, context) {
        injection_handler_1.InjectorHandler.injectContext(constructor, context);
        const instance = (params ? new constructor(...params) : new constructor());
        injection_handler_1.InjectorHandler.removeContext(constructor);
        return instance;
    }
    factory(factory) {
        this.iocFactory = (context) => {
            const instance = factory(context);
            injection_handler_1.InjectorHandler.injectContext(instance, context);
            return instance;
        };
        if (this.iocScope) {
            this.iocScope.reset(this.source);
        }
        return this;
    }
    scope(scope) {
        if (this.iocScope && this.iocScope !== scope) {
            this.iocScope.finish(this.source);
        }
        this.iocScope = scope;
        if (this.iocScope) {
            this.iocScope.init(this.source);
        }
        return this;
    }
    withParams(...paramTypes) {
        this.paramTypes = paramTypes;
        return this;
    }
    instrumentConstructor() {
        const newConstructor = injection_handler_1.InjectorHandler.instrumentConstructor(this.source);
        this.decoratedConstructor = newConstructor;
        this.source.constructor = newConstructor;
        return this;
    }
    getInstance(context) {
        if (!this.iocScope) {
            this.scope(model_1.Scope.Local);
        }
        if (this.decoratedConstructor) {
            return this.getContainerManagedInstance(context);
        }
        else {
            return this.iocScope.resolve(this.iocFactory, this.source, context);
        }
    }
    getContainerManagedInstance(context) {
        injection_handler_1.InjectorHandler.unblockInstantiation(this.source);
        const instance = this.iocScope.resolve(this.iocFactory, this.source, context);
        injection_handler_1.InjectorHandler.blockInstantiation(this.source);
        return instance;
    }
    getParameters(context) {
        if (this.paramTypes) {
            return this.paramTypes.map(paramType => this.instanceFactory(paramType, context));
        }
        return null;
    }
}
exports.IoCBindConfig = IoCBindConfig;
//# sourceMappingURL=container-binding-config.js.map